#!/usr/bin/env python
#-*- coding:utf-8 -*-

"""
supymemo importer
Copyright (c) 2012 Clovis Fabricio Costa - All Rights Reserved

Licensed under the Academic Free License version 3.0
You should have received a full copy of the license in the LICENSE file
together with this source code. If not, you can download a copy in
http://www.opensource.org/licenses/academic.php

This program reads questions/answers from some websites and generates a file
suitable for use with the Supermemo application.

Supported websites:
http://www.studystack.com
http://quizlet.com
http://flashcardexchange.com

author: nosklo
"""

import optparse
import lxml.html
import lxml.etree
import re
import sys
import json


def remove_html_entities(text):
    return lxml.html.fromstring('<p>' + text + '</p>').text_content()

def extract_studystack_qa(studystack_id):
    url = 'http://www.studystack.com/servlet/xmlStudyStack/supymemo.xmlx?studyStackId=%d&charset=UTF-8'
    doc = lxml.etree.parse(url % int(studystack_id))
    expr = "./Flap[@name=$name]/text()"
    for card in doc.xpath('//Card'):
        yield {
            'word': card.xpath(expr, name='Question')[0].replace('\r', '\n'),
            'definition': card.xpath(expr, name='Answer')[0].replace('\r', '\n'),
        }

_quizlet_re = re.compile(r'//\s*<!\[CDATA\[\s*words\s*=\s*(.*)\s*;\s*//\s*\]\]>', 
    re.DOTALL | re.MULTILINE)
def extract_quizlet_qa(quizlet_id):
    doc = lxml.html.parse('http://quizlet.com/%d/export/' % int(quizlet_id))
    for text in doc.xpath('//script/text()'):
        m = _quizlet_re.search(text)
        if m:
            return json.loads(m.group(1))
    else:
        raise Exception('ERROR: data not found for http://quizlet.com/%d '
            'or the site changed format.' % int(quizlet_id))

_flashcardexchange_re = re.compile(
    r'var\s+Flashex\s+=\s+\{\s+flashcards\s*:\s*(\[.*?\])\s*\}',
    re.DOTALL | re.MULTILINE)
def extract_flashcardexchange_qa(fce_id):
    url = 'http://www.flashcardexchange.com/flashcards/list/%d'
    doc = lxml.html.parse(url % int(fce_id))
    for text in doc.xpath('//script/text()'):
        m = _flashcardexchange_re.search(text)
        if not m:
            continue
        for element in json.loads(m.group(1)):
            yield {
                'word': remove_html_entities(element['question']),
                'definition': remove_html_entities(element['answer']),
            }
        return
    else:
        raise Exception('ERROR: data not found for '
            'http://www.flashcardexchange.com/flashcards/view/%d or the '
            'site changed format.' % int(fce_id))
            

parser = optparse.OptionParser(usage='%prog [options] card_id1 [card_id2...]')
parser.add_option("-r", "--reverse", action="store_true", dest="reverse", 
    default=False, help="reverse questions and answers")

pgroup = optparse.OptionGroup(parser, 'Websites', 
    'You have to choose one of those options to fetch the cards from:')

pgroup.add_option("-s", "--studystack", action="store_const", dest="function", 
    default=None, help="Pull http://www.studystack.com cards", 
    const=extract_studystack_qa)
pgroup.add_option("-q", "--quizlet", action="store_const", dest="function", 
    default=None, help="Pull http://quizlet.com cards", 
    const=extract_quizlet_qa)
pgroup.add_option("-f", "--flashcardexchange", action="store_const", dest="function", 
    default=None, help="Pull http://flashcardexchange.com cards", 
    const=extract_flashcardexchange_qa)
parser.add_option_group(pgroup)

if __name__ == '__main__':
    (options, args) = parser.parse_args()
    if not options.function:
        parser.print_help()
        print '\nERROR: you must choose one of the websites.'
        sys.exit()
    if not args:
        parser.print_help()
        print ("\nERROR: You have to pass the number id of the card sets\n"
            "(which is probably in the url, i.e. http://quizlet.com/99999999)\n"
            "Example:\n"
            "python %s --quizlet 1068559 1068747" % (__file__,))

    for cardset_id in args:
        output = open(cardset_id + '.txt', 'w')

        for row in options.function(cardset_id):
            row = tuple(row[field].replace('\r\n', '\x02').replace('\n', '\x02')
                for field in ('word', 'definition'))
            output.write("Q: %s\nA: %s\n\n" % row)
            if options.reverse:
                output.write("Q: %s\nA: %s\n\n" % row[::-1])

        output.close()
